use std::collections::HashMap;

use serde::{Deserialize, Serialize};

#[derive(Serialize)]
pub struct ResourceRequest {
    request_origin: String,
    resource_types: Vec<String>,
}

#[derive(Deserialize,PartialEq,Debug)]
pub struct Resource {
    r#type: String,
    blocked_in: HashMap<String, bool>,
    protocol: String,
    address: String,
    port: u16,
    fingerprint: String,
    #[serde(rename="or-addresses")]
    or_addresses: Option<Vec<String>>,
    distribution: String,
    flags: Option<HashMap<String,bool>>,
    params: Option<HashMap<String,String>>,
}

#[derive(Deserialize)]
pub struct ResourceDiff {
    new: Option<HashMap<String,Vec<Resource>>>,
    changed: Option<HashMap<String,Vec<Resource>>>,
    gone: Option<HashMap<String,Vec<Resource>>>,
    full_update: bool,
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn serialize_resource_request() {
        let req = ResourceRequest {
            request_origin: String::from("https"),
            resource_types: vec![String::from("obfs2"), String::from("scramblesuit")],
        };
        let json = serde_json::to_string(&req).unwrap();
        assert_eq!(json, "{\"request_origin\":\"https\",\"resource_types\":[\"obfs2\",\"scramblesuit\"]}")
    }

    #[test]
    fn deserialize_resource() {
        let mut flags = HashMap::new();
        flags.insert(String::from("fast"), true);
        flags.insert(String::from("stable"), true);
        let mut params = HashMap::new();
        params.insert(String::from("password"), String::from("ABCDEFGHIJKLMNOPQRSTUVWXYZ234567"));
        let bridge = Resource {
            r#type: String::from("scramblesuit"),
            blocked_in: HashMap::new(),
            protocol: String::from("tcp"),
            address: String::from("216.117.3.62"),
            port: 63174,
            fingerprint: String::from("BE84A97D02130470A1C77839954392BA979F7EE1"),
            or_addresses: None,
            distribution: String::from("https"),
            flags: Some(flags),
            params: Some(params),
        };

        let data = r#"
            {
                "type": "scramblesuit",
                "blocked_in": {},
                "protocol": "tcp",
                "address": "216.117.3.62",
                "port": 63174,
                "fingerprint": "BE84A97D02130470A1C77839954392BA979F7EE1",
                "or-addresses": null,
                "distribution": "https",
                "flags": {
                    "fast": true,
                    "stable": true
                },
                "params": {
                    "password": "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567"
                }
            }"#;
        let res: Resource = serde_json::from_str(data).unwrap();
        assert_eq!(bridge, res);
    }

    #[test]
    fn deserialize_resource_diff() {
        let data = r#"
            {
                "new": {
                    "obfs2": [
                        {
                            "type": "obfs2",
                            "blocked_in": {},
                            "protocol": "tcp",
                            "address": "176.247.216.207",
                            "port": 42810,
                            "fingerprint": "10282810115283F99ADE5CFE42D49644F45D715D",
                            "or-addresses": null,
                            "distribution": "https",
                            "flags": {
                                "fast": true,
                                "stable": true,
                                "running": true,
                                "valid": true
                            }
                        },
                        {
                            "type": "obfs2",
                            "blocked_in": {},
                            "protocol": "tcp",
                            "address": "133.69.16.145",
                            "port": 58314,
                            "fingerprint": "BE84A97D02130470A1C77839954392BA979F7EE1",
                            "or-addresses": null,
                            "distribution": "https",
                            "flags": {
                                "fast": true,
                                "stable": true,
                                "running": true,
                                "valid": true
                            }
                        }
                    ],
                    "scramblesuit": [
                        {
                            "type": "scramblesuit",
                            "blocked_in": {},
                            "protocol": "tcp",
                            "address": "216.117.3.62",
                            "port": 63174,
                            "fingerprint": "BE84A97D02130470A1C77839954392BA979F7EE1",
                            "or-addresses": null,
                            "distribution": "https",
                            "flags": {
                                "fast": true,
                                "stable": true,
                                "running": true,
                                "valid": true
                            },
                            "params": {
                                "password": "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567"
                            }
                        }
                    ]
                },
                "changed": null,
                "gone": null,
                "full_update": true
            }"#;
        let _diff: ResourceDiff = serde_json::from_str(data).unwrap();
    }
}
